Package: fAssets
Title: Rmetrics - Analysing and Modelling Financial Assets
Date: 2023-04-20
Version: 4023.85
Authors@R: c( person("Diethelm", "Wuertz", role = "aut"), person("Tobias", "Setz", role = "aut"),
              person("Yohan", "Chalabi", role = "aut"),
	      person("Stefan", "Theussl", role = c("aut", "cre"), email = "Stefan.Theussl@R-project.org", comment = c(ORCID = "0000-0002-6523-4620")) )
Description: A collection of functions to manage, to investigate and to analyze data sets of financial 
  assets from different points of view.
Depends: R (>= 2.15.1), timeDate, timeSeries, fBasics
Imports: fMultivar, robustbase, MASS, sn, ecodist, mvnormtest, energy,
        grDevices, graphics, stats
Suggests: methods, mnormt, RUnit
License: GPL (>= 2)
URL: https://r-forge.r-project.org/projects/rmetrics/
NeedsCompilation: no
Packaged: 2023-04-21 19:38:00 UTC; parallels
Author: Diethelm Wuertz [aut],
  Tobias Setz [aut],
  Yohan Chalabi [aut],
  Stefan Theussl [aut, cre] (<https://orcid.org/0000-0002-6523-4620>)
Maintainer: Stefan Theussl <Stefan.Theussl@R-project.org>
Repository: CRAN
Date/Publication: 2023-04-24 14:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 03:08:43 UTC; unix
